@@ -1,4 +1,4 @@
-# SPDX-FileCopyrightText: 2009 Jordan Terell (blog.jordanterrell.com)
+# SPDX-FileCopyrightText: 2009 Jordan Terrell (blog.jordanterrell.com)
 # SPDX-FileCopyrightText: 2020 Brent Rubell for Adafruit Industries
 # SPDX-FileCopyrightText: 2021 Patrick Van Oosterwijck @ Silicognition LLC
 #
@@ -13,6 +13,17 @@ Pure-Python implementation of Jordan Terrell's DHCP library v0.3
 * Author(s): Jordan Terrell, Brent Rubell
 
 """
+from __future__ import annotations
+
+try:
+    from typing import TYPE_CHECKING, Optional, Union, Tuple, Sequence
+
+    if TYPE_CHECKING:
+        from adafruit_wiznet5k.adafruit_wiznet5k import WIZNET5K
+except ImportError:
+    pass
+
+
 import gc
 import time
 from random import randint
@@ -79,20 +90,25 @@ _BUFF = bytearray(318)
 
 
 class DHCP:
-    """W5k DHCP Client implementation.
-
-    :param eth: Wiznet 5k object
-    :param list mac_address: Hardware MAC.
-    :param str hostname: The desired hostname, with optional {} to fill in MAC.
-    :param int response_timeout: DHCP Response timeout.
-    :param bool debug: Enable debugging output.
-
-    """
+    """W5k DHCP Client implementation."""
 
     # pylint: disable=too-many-arguments, too-many-instance-attributes, invalid-name
     def __init__(
-        self, eth, mac_address, hostname=None, response_timeout=30, debug=False
-    ):
+        self,
+        eth: WIZNET5K,
+        mac_address: Sequence[Union[int, bytes]],
+        hostname: Optional[str] = None,
+        response_timeout: float = 30.0,
+        debug: bool = False,
+    ) -> None:
+        """
+        :param adafruit_wiznet5k.WIZNET5K eth: Wiznet 5k object
+        :param Sequence[Union[int, bytes]] mac_address: Hardware MAC address.
+        :param Optional[str] hostname: The desired hostname, with optional {} to fill
+            in the MAC address, defaults to None.
+        :param float response_timeout: DHCP Response timeout in seconds, defaults to 30.
+        :param bool debug: Enable debugging output.
+        """
         self._debug = debug
         self._response_timeout = response_timeout
 
@@ -137,12 +153,18 @@ class DHCP:
         )
 
     # pylint: disable=too-many-statements
-    def send_dhcp_message(self, state, time_elapsed, renew=False):
-        """Assemble and send a DHCP message packet to a socket.
+    def send_dhcp_message(
+        self,
+        state: int,
+        time_elapsed: float,
+        renew: bool = False,
+    ) -> None:
+        """
+        Assemble and send a DHCP message packet to a socket.
 
         :param int state: DHCP Message state.
         :param float time_elapsed: Number of seconds elapsed since DHCP process started
-        :param bool renew: Set True for renew and rebind
+        :param bool renew: Set True for renew and rebind, defaults to False
         """
         _BUFF[:] = b"\x00" * len(_BUFF)
         # OP
@@ -238,9 +260,12 @@ class DHCP:
         self._sock.send(_BUFF)
 
     # pylint: disable=too-many-branches, too-many-statements
-    def parse_dhcp_response(self):
+    def parse_dhcp_response(
+        self,
+    ) -> Union[Tuple[int, bytes], Tuple[int, int]]:
         """Parse DHCP response from DHCP server.
-        Returns DHCP packet type.
+
+        :return Union[Tuple[int, bytes], Tuple[int, int]]: DHCP packet type.
         """
         # store packet in buffer
         _BUFF = self._sock.recv()
@@ -347,10 +372,12 @@ class DHCP:
         return msg_type, xid
 
     # pylint: disable=too-many-branches, too-many-statements
-    def _dhcp_state_machine(self):
-        """DHCP state machine without wait loops to enable cooperative multi tasking
+    def _dhcp_state_machine(self) -> None:
+        """
+        DHCP state machine without wait loops to enable cooperative multitasking.
         This state machine is used both by the initial blocking lease request and
-        the non-blocking DHCP maintenance function"""
+        the non-blocking DHCP maintenance function.
+        """
         if self._eth.link_status:
             if self._dhcp_state == STATE_DHCP_DISCONN:
                 self._dhcp_state = STATE_DHCP_START
@@ -485,7 +512,7 @@ class DHCP:
                 self._sock.close()
                 self._sock = None
 
-    def request_dhcp_lease(self):
+    def request_dhcp_lease(self) -> bool:
         """Request to renew or acquire a DHCP lease."""
         if self._dhcp_state in (STATE_DHCP_LEASED, STATE_DHCP_WAIT):
             self._dhcp_state = STATE_DHCP_START
@@ -495,6 +522,6 @@ class DHCP:
 
         return self._dhcp_state == STATE_DHCP_LEASED
 
-    def maintain_dhcp_lease(self):
+    def maintain_dhcp_lease(self) -> None:
         """Maintain DHCP lease"""
         self._dhcp_state_machine()
